;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Bony Beetle (sprite 31), by imamelia
;;
;; This is a disassembly of sprite 31 in SMW, the Bony Beetle.
;;
;; Uses first extra bit: NO
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Tilemap:
db $8C,$AA,$86,$84

SlopeEor:
db $08,$F8,$02,$03,$04,$04,$04,$04,$04,$04,$04,$04

XDisp2:		; displacement of the second crumbled tile depending on sprite direction
db $08,$F8	;

!CrumbledTile1 = $2E	; the fully-crumbled tile (the 16x16 tile 8 pixels to the left is also used)
!CrumbledTile2 = $48	; the half-crumbled tile (the 16x16 tile 8 pixels to the left is also used)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc

JSR FacePlayer	; face the player initially

RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR BonyBeetleMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

NotCrumbled1:
JMP NotCrumbled

BonyBeetleMain:

LDA $14C8,x	;
CMP #$08		; if the sprite is in normal status...
BEQ NormalRt	; run the standard code
ASL $15F6,x	; if not...
SEC		; set bit 7 of $15F6,x (sprite palette and GFX page) so that the sprite graphics will be Y-flipped
ROR $15F6,x	; (actually, it doesn't matter, since $15F6,x is never referenced anyway)
JMP SkipToGFX	; and go directly to the GFX routine

NormalRt:		;

LDA $1534,x	; if the crumbled flag is set...
BEQ NotCrumbled1	; go to that part of the code

JSL $0190B2	; part of the setup for the crumbled GFX (shared GFX routine, but the values are changed anyway)

LDY $1540,x	; if the stun timer is not set...
BNE NoFace1	;
STZ $1534,x	; clear the crumbled flag
PHY		; preserve the OAM index
JSR FacePlayer	;
PLY		; pull back the OAM index

NoFace1:

LDA #!CrumbledTile2	; the half-crumbled tile
CPY #$10			; if the sprite is just about to get up...
BCC StoreCrumbledTile	;
CPY #$F0			; or if it just got squished...
BCS StoreCrumbledTile	; use the half-crumbled tile
LDA #!CrumbledTile1	; if not, use the fully-crumbled tile
StoreCrumbledTile:		;
LDY $15EA,x		; load the sprite OAM index back into Y
STA $0302,y		; set the tile number

TYA			;
CLC			;
ADC #$04		; increment the OAM index by 4
STA $15EA,x		;

PHX			; preserve the sprite index
LDA $157C,x		;
TAX			; get the sprite direction into X
LDA $0300,y		;
CLC			;
ADC XDisp2,x		; set the tile X displacement depending on the sprite direction
PLX			;
STA $0304,y		;

LDA $0301,y		;
STA $0305,y		; set the same Y displacement
LDA $0303,y		;
STA $0307,y		; and tile properties for both tiles

LDA $0302,y		; tile number of the first tile
DEC 			; the second tile always has a tile number one less than the first
STA $0306,y		; set the tile number of the second tile

LDA $1540,x		; if the stun timer is set...
BEQ NoShakeDisp		;
CMP #$40			; and it is less than 40...
BCS NoShakeDisp		;
LSR #2			; right-shift it twice
PHP			; save the processor flags
LDA $0300,y		;
ADC #$00		; this will actually add 01 if bit 2 of the stun timer was set,
STA $0300,y		; since that bit went into the carry flag
PLP			; we needed to save those processor flags just in case that ADC unset the carry flag
LDA $0304,y		;
ADC #$00		; the same is true for the second tile: it will be shifted 00 or 01 pixels to the right
STA $0304,y		;

NoShakeDisp:

LDY #$02			; the tiles were 16x16
LDA #$01			; and 2 tiles were drawn
JSL $81B7B3		; finish the OAM write
JSL $81802A		; update sprite position
LDA $1588,x		;
AND #$04		; if the sprite is on ground...
BEQ EndCrumbledCode	;
STZ $AA,x		; set its Y speed
STZ $B6,x			; and its X speed to zero

EndCrumbledCode:		;
RTS			;

NotCrumbled:

LDA $9D			; if sprites are locked...
ORA $163E,x		; (not sure what this does...$163E,x isn't referenced or modifed anywhere else)
BEQ Continue1		;
JMP Skip1			;

Continue1:

LDY $157C,x		;
LDA SlopeEor,y		; not sure what the purpose of this is...
EOR $15B8,x		; EOR it with the value of the slope the sprite is standing on
ASL			;
LDA SlopeEor,y		; now load the data itself
BCC StoreXSpeed		; if bit 7 of the data/slope EOR was set, store the X speed directly
CLC			;
ADC $15B8,x		; if not, add the value from the slope first
StoreXSpeed:		;
STA $B6,x			;

LDA $1540,x		; if the stun timer is set...
BNE ZeroXSpeed		; set the sprite X speed to 0
TYA			; if not...
INC A			; transfer the sprite direction to A and then increment it
AND $1588,x		; AND it with the sprite direction status
AND #$03		; if the sprite is touching a wall...
BEQ NoZeroXSpeed		;
ZeroXSpeed:		;
STZ $B6,x			;
NoZeroXSpeed:		;

LDA $1588,x		;
AND #$08		; if the sprite is touching the ceiling...
BEQ NoZeroYSpeed		;
STZ $AA,x		; set the sprite Y speed to 0
NoZeroYSpeed:		;

JSR SubOffscreenX0		; offscreen processing
JSL $01802A		; update sprite position

; removed sprite number check:
;CODE_01E4FC:        B5 9E         LDA RAM_SpriteNum,X       
;CODE_01E4FE:        C9 31         CMP.B #$31                
;CODE_01E500:        D0 1C         BNE CODE_01E51E           

LDA $1540,x	; if the (spike) timer is not set...
BEQ GroundCheck	; branch

LDY #$00		; Y = 00
CMP #$70		; if the spike timer is 70 or greater...
BCS SetFrame	; then the frame will be 00
INY #2		; increment Y twice
CMP #$08		; if the timer is less than 08...
BCC SetFrame	;
CMP #$68		; or greater than 67...
BCS SetFrame	; Y = 02
INY		; else, Y = 03             

SetFrame:

TYA		; transfer the frame value from Y to A
STA $1602,x	; set the frame number
BRA NotOnGround

GroundCheck:

LDA $1588,x	;
AND #$04	; if the sprite is not on the ground...
BEQ NotOnGround	; skip the next part of code

JSR SetSomeYSpeed		;
JSR SetAnimationFrame	;

Skip2:			;

LDA $1570,x		;
AND #$7F		; every 80 frames...
BNE NoUpdate		;

JSR FacePlayer		; turn to face the player
BRA NoUpdate

NotOnGround:

STZ $1570,x		;

LDA $C2,x		;
BNE NoUpdate		;
INC $C2,x		;

JSR FlipSpriteDir		; flip the sprite's direction
JSL $018022		; update sprite X position without gravity...
JSL $018022		; twice

NoUpdate:

; removed sprite number check:
;CODE_01E57B:        B5 9E         LDA RAM_SpriteNum,X
;CODE_01E57D:        C9 31         CMP.B #$31
;CODE_01E57F:        D0 17         BNE CODE_01E598

LDA $13		;
LSR		; every other frame...
BCC NoInc	;
INC $1528,x	; increment some misc. table
NoInc:		;
LDA $1528,x	;
BNE Skip1		;
INC $1528,x	;
LDA #$A0		; reset the spike timer
STA $1540,x	;

Skip1:

JSR Interact	; player interaction code
JSL $018032	; interact with other sprites
JSR MaybeFlip	;

SkipToGFX:

JSR DryBonesGFX	; draw the sprite

Return00:		;
RTS		;

Interact:

JSL $01A7DC	; check player/sprite contact
BCC Return00	; if there is none, just return

LDA $D3		;
CLC		;
ADC #$14	; if the player is less than 14 pixels above the sprite 
CMP $D8,x	;
BPL SpriteWins	; hurt the player

LDA $1697	; check the number of consecutive enemies stomped
BNE PlayerWins	;
LDA $7D		; if the player is moving upward...
BMI SpriteWins	;

; removed sprite number check:
;CODE_01E5DB:        B5 9E         LDA RAM_SpriteNum,X
;CODE_01E5DD:        C9 31         CMP.B #$31
;CODE_01E5DF:        D0 0A         BNE CODE_01E5EB

LDA $1540,x	; check the spike timer
SEC		;
SBC #$08		; if it is between 08 and 67...
CMP #$60		;
BCC SpriteWins	; then the sprite hurts the player

PlayerWins:

PHK		;
PER $0006	;
PEA $8020	;
JML $81AB46	; this subroutine handles the consecutive enemies stomped counter
JSL $81AB99	; display contact GFX

LDA #$07		;
STA $1DF9	; play the crumbling sound effect

JSL $81AA33	; boost the player's Y speed

INC $1534,x	; set the crumbled flag
LDA #$FF		;
STA $1540,x	; set the time until the sprite gets back up

RTS

SpriteWins:

JSL $80F5B7	; hurt the player
LDA $1497	; unless the player is flashing invincible...
BNE Return01	;
JSR FacePlayer	; turn to face him/her

Return01:		;
RTS

MaybeFlip:	; ripped from $019089

LDA $157C,x	;
INC		;
AND $1588,x	; if the sprite is touching an object in the direction that it is facing...
AND #$03	;
BEQ Return01	;

JSR FlipSpriteDir	; flip it

RTS		;

FlipSpriteDir:	; ripped from $019098

LDA $15AC,x	; if the turn timer is already set...
BNE Return01	; return

LDA #$08		;
STA $15AC,x	; set the turn timer
LDA $B6,x	;
EOR #$FF		; invert the sprite X speed
INC		;
STA $B6,x		;
LDA $157C,x	;
EOR #$01		; flip the sprite direction
STA $157C,x	;

RTS

FacePlayer:

JSR SubHorzPos	;
TYA		;
STA $157C,x	;
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DryBonesGFX:

LDA $157C,x	;
PHA		; preserve the sprite direction
LDY $15AC,x	;
BEQ JustDraw	;
CPY #$05		; if the turn timer is 05 or greater...
BCC JustDraw	;
EOR #$01		; flip the sprite direction before drawing it
STA $157C,x	;

JustDraw:		;

JSR MainGFXRt	;

PLA		;
STA $157C,x	;

MainGFXRt:

; removed sprite number check:
;CODE_03C3DA:        B5 9E         LDA RAM_SpriteNum,X       
;CODE_03C3DC:        C9 31         CMP.B #$31                
;CODE_03C3DE:        F0 CE         BEQ CODE_03C3AE           

JSR GetDrawInfo	;

LDA $157C,x	;
STA $02		;

LDA $1602,x	;
TAX		;
LDA Tilemap,x	; set the sprite tilemap
STA $0302,y	;

LDX $15E9	;
LDA $00		;
STA $0300,y	; no X displacement
LDA $01		;
STA $0301,y	; no Y displacement

LDA $157C,x	;
LSR		; if the sprite is facing right...
LDA $15F6,x	;
BCS NoXFlip	; X-flip it
EOR #$40		;
NoXFlip:		;
ORA $64		;
STA $0303,y	;

TYA		;
LSR #2		;
TAY		;

LDA #$02		;
ORA $15A0,x	;
STA $0460,y	; set the tile size

PHK		;
PER $0006	;
PEA $8020	;
JML $81A3DF	; set up some stuff in OAM

RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; miscellaneous standard subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreenX0:
LDA #$00
;BRA SubOffscreenMain
;SubOffscreenX1:
;LDA #$02
;BRA SubOffscreenMain
;SubOffscreenX2:
;LDA #$04
;BRA SubOffscreenMain
;SubOffscreenX3:
;LDA #$06
;BRA SubOffscreenMain
;SubOffscreenX4:
;LDA #$08
;BRA SubOffscreenMain
;SubOffscreenX5:
;LDA #$0A
;BRA SubOffscreenMain
;SubOffscreenX6:
;LDA #$0C
;BRA SubOffscreenMain
;SubOffscreenX7:
;LDA #$0E

SubOffscreenMain:

STA $03

JSR SubIsOffscreen
BEQ Return2

LDA $5B
LSR
BCS VerticalLevel
LDA $D8,x
CLC
ADC #$50
LDA $14D4,x
ADC #$00
CMP #$02
BPL EraseSprite
LDA $167A,x
AND #$04
BNE Return2
LDA $13
AND #$01
ORA $03
STA $01
TAY
LDA $1A
CLC
ADC Table4,y
ROL $00
CMP $E4,x
PHP
LDA $1B
LSR $00
ADC Table5,y
PLP
SBC $14E0,x
STA $00
LSR $01
BCC Label20
EOR #$80
STA $00
Label20:
LDA $00
BPL Return2

EraseSprite:
LDA $14C8,x
CMP #$08
BCC KillSprite
LDY $161A,x
CPY #$FF
BEQ KillSprite
LDA #$00
STA $1938,y
KillSprite:
STZ $14C8,x
Return2:
RTS

VerticalLevel:

LDA $167A,x
AND #$04
BNE Return2
LDA $13
LSR
BCS Return2
AND #$01
STA $01
TAY
LDA $1C
CLC
ADC Table3,y
ROL $00
CMP $D8,x
PHP
LDA $1D
LSR $00
ADC Table6,y
PLP
SBC $14D4,x
STA $00
LDY $02
BEQ Label22
EOR #$80
STA $00
Label22:
LDA $00
BPL Return2
BMI EraseSprite

SubIsOffscreen:
LDA $15A0,x
ORA $186C,x
RTS

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS

SubHorzPos:

LDY #$00
LDA $94
SEC
SBC $E4,x
STA $0F
LDA $95
SBC $14E0,x
BPL $01
INY
RTS

SetSomeYSpeed:	; ripped from $019A04

LDA $1588,x
BMI $07
LDA #$00
LDY $15B8,x
BEQ $02
LDA #$18
STA $AA,x
RTS

SetAnimationFrame:	; ripped from $018E5E

INC $1570,x
LDA $1570,x
LSR #3
AND #$01
STA $1602,x
RTS














